--[[
**********************************************************************
MagicDKP - a raid loot and DKP tracker.
**********************************************************************
This file is part of MagicDKP, a World of Warcraft Addon

MagicDKP is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

MagicDKP is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with MagicDKP.  If not, see <http://www.gnu.org/licenses/>.
**********************************************************************
]]

local MINOR_VERSION = tonumber(("$Revision: 69897 $"):match("%d+"))

MagicDKP = LibStub("AceAddon-3.0"):NewAddon("MagicDKP", "AceConsole-3.0",
					    "AceEvent-3.0", "AceTimer-3.0", "LibLogger-1.0")
MagicDKP.MAJOR_VERSION = "MagicDKP-1.0"
MagicDKP.MINOR_VERSION = MINOR_VERSION

local MagicComm = LibStub("MagicComm-1.0")
local GUI = LibStub("AceGUI-3.0")
local BabbleBoss = LibStub("LibBabble-Boss-3.0") 
local BR = BabbleBoss:GetReverseLookupTable()
local BL = BabbleBoss:GetLookupTable()
BabbleBoss = nil

MagicDKPDB = {}

local MagicDKP = MagicDKP
local L = LibStub("AceLocale-3.0"):GetLocale("MagicDKP", false)

local UnitGUID = UnitGUID
local sub = string.sub
local UnitClassification = UnitClassification
local table_insert = table.insert
local loggingDeaths = false

local defaultOptions = {
   profile = {
      logLevel = MagicDKP.logLevels.INFO,
      raidsize = "25",
      trackwipe = false,
      wipepercentage = 50,
   }
}

local db, zones, currentZone, currentRaid, currentEvent, currentBoss
local playerName = UnitName("player")

local networkData = {}

local function SetNetworkData(cmd, data, misc1, misc2, misc3, misc4)
   networkData.cmd = cmd
   networkData.data = data
   networkData.misc1 = misc1
   networkData.misc2 = misc2
   networkData.misc3 = misc3
   networkData.misc4 = misc4
end
      

-- Dummy debug ftw.
function MagicDKP:get()
   return {
      zone = currentZone,
      raid = currentRaid,
      event = currentEvent
   }
end

local function UpdateUpvalues()
   db = MagicDKP.db.profile         -- profile data
   zones = MagicDKPDB.zones     -- raid zones / boss mobs
   items = MagicDKPDB.items     -- item database
   events = MagicDKPDB.events   -- custom events
   raids = MagicDKPDB.raids     -- raids
   players = MagicDKPDB.players -- player db
end

function MagicDKP:OnInitialize()
   self:SetPerformanceMode(true) -- performance logging
 
   MagicDKP.db = LibStub("AceDB-3.0"):New("MagicDKPConfigDB", defaultOptions, "Default")
   self.db.RegisterCallback(self, "OnProfileChanged", "OnProfileChanged")
   self.db.RegisterCallback(self, "OnProfileCopied", "OnProfileChanged")
   self.db.RegisterCallback(self, "OnProfileDeleted","OnProfileChanged")
   self.db.RegisterCallback(self, "OnProfileReset", "OnProfileChanged")

   MagicDKPDB.zones    = MagicDKPDB.zones    or {}
   MagicDKPDB.items    = MagicDKPDB.items    or MagicDKP.defaultItems
   MagicDKPDB.events   = MagicDKPDB.events   or MagicDKP.defaultEvents  
   MagicDKPDB.raids    = MagicDKPDB.raids    or {}
   MagicDKPDB.players  = MagicDKPDB.players  or {}

   MagicDKP.defaultItems  = nil 
   MagicDKP.defaultEvents = nil 
   
   UpdateUpvalues()

   self:SetLogLevel(db.logLevel)
   self:AddDefaultZones() -- Populate MagicDKPDB with the default zones
   self:BuildConfiguration()   
end



local currentNoteItem = {}
 noteItemQueue = {}
local itemNoteText = L["Enter value for %s, looted by %s."]
local itemLootDialog

local function CheckQueuedItem()
   local count =  #noteItemQueue
   local item = count > 0 and noteItemQueue[count]
   if item then
      MagicDKP:AddItemDialog(item.looter, item.itemid, item.value, item.link, item.count)
      noteItemQueue[count] = nil
   else
      currentNoteItem.itemid = nil
      currentNoteItem.looter = nil
      currentNoteItem.value = nil
      currentNoteItem.count = nil
      itemLootDialog:Hide()
   end
end

local function QueueItem(looter, itemid, value, link, count)
   local queueSize = #noteItemQueue
   if queueSize > 0 or currentNoteItem.itemid then
      noteItemQueue[#noteItemQueue+1] = {
	 looter = looter,
	 itemid = itemid,
	 value = value or 0,
	 link = link,
	 count = count 
      }
   else
      MagicDKP:AddItemDialog(looter, itemid, value or 0, link, count)
   end
   
end

function MagicDKP:AddItem(looter, itemid, value, count)
   if self.trace then
      self:trace("Adding item %sx%d for %s at value %d.", items[itemid].itemInfo[2], count, looter, value)
   end

   if currentRaid then
      local newitem = {
	 itemid = itemid,
	 looter = looter,
	 value = value,
	 count = count
      }
      if currentEvent then
	 currentEvent.items[#currentEvent.items+1] = newitem
      else
	 local key,name = self:GetZoneName(currentZone.name)
	 if not currentRaid.events[name] then
	    currentRaid.events[name] = {
	       attendees = { [UnitName("player")] = true },
	       eventTime = self:GetGameTimeAsUnix(), -- set event time to time of first looted item
	       items = {}
	    }
	 end
	 currentRaid.events[name].items[#currentRaid.events[name].items+1] = newitem
      end

      for id,data in pairs(raids) do
	 if data == currentRaid then
	    self:AddRaidConfig(id, build)
	    break;
	 end
      end
      self:NotifyChange()
   end  
end

function MagicDKP:AddItemDialog(looter, itemid, value, link, count)
   currentNoteItem.itemid = itemid
   currentNoteItem.looter = looter
   currentNoteItem.value = value
   currentNoteItem.count = count
   
   if not itemLootDialog then
      itemLootDialog = GUI:Create("MDKP_Dialog")
      itemLootDialog:SetTitle(L["Add Item"])
      itemLootDialog:SetLayout("Flow")

      itemLootDialog.ok = GUI:Create("Button")
      itemLootDialog.cancel = GUI:Create("Button")
      itemLootDialog.bank = GUI:Create("Button")
      itemLootDialog.de = GUI:Create("Button")

      itemLootDialog:AddChild(itemLootDialog.ok)
      itemLootDialog:AddChild(itemLootDialog.cancel)
      itemLootDialog:AddChild(itemLootDialog.bank)
      itemLootDialog:AddChild(itemLootDialog.de)

      itemLootDialog.ok:SetText(L["Ok"])
      itemLootDialog.cancel:SetText(L["Ignore"])
      itemLootDialog.bank:SetText(L["Bank"])
      itemLootDialog.de:SetText(L["Disenchant"])

      local onAccept = function()
			  local val = itemLootDialog:GetValue()
			  MagicDKP:AddItem(currentNoteItem.looter, currentNoteItem.itemid, val, currentNoteItem.count)
			  items[currentNoteItem.itemid].value = val
			  CheckQueuedItem()
		       end
      
      itemLootDialog:SetCallback("OnAccept", onAccept)
      itemLootDialog.ok:SetCallback("OnClick", onAccept)
      itemLootDialog.cancel:SetCallback("OnClick",
				    function()
				       items[currentNoteItem.itemid].ignore = true
				       items[currentNoteItem.itemid].value = nil
				       items[currentNoteItem.itemid].bank = nil
				       CheckQueuedItem()
				    end
				 )
      itemLootDialog.bank:SetCallback("OnClick",
				    function()
				       items[currentNoteItem.itemid].bank = true
				       items[currentNoteItem.itemid].value = nil
				       MagicDKP:AddItem("bank", currentNoteItem.itemid, 0, currentNoteItem.count)
				       CheckQueuedItem()
				    end
				 )
      itemLootDialog.de:SetCallback("OnClick",
				    function()
				       MagicDKP:AddItem("disenchant", currentNoteItem.itemid, 0, currentNoteItem.count)
				       CheckQueuedItem()
				    end
				 )
      itemLootDialog.ok:SetWidth(80)
      itemLootDialog.cancel:SetWidth(80)
      itemLootDialog.bank:SetWidth(80)
      itemLootDialog.de:SetWidth(125)
   end

   
   itemLootDialog:SetText(itemNoteText:format(link, looter ))
   itemLootDialog:SetValue(value)

   itemLootDialog:Show()
   return itemLootDialog
   
end


function MagicDKP:OnEnable()
   local popup = _G.StaticPopupDialogs
   if type(popup) ~= "table" then
      popup = {}
   end
   if type(popup["MDKPAttendance"]) ~= "table" then
      popup["MDKPAttendance"] = {
	 text = "",
	 button1 = TEXT(YES),
	 button2 = TEXT(NO),
	 timeout = 0,
	 whileDead = 1,
	 hideOnEscape = 0,
	 OnAccept = function() self:TakeAttendance() end
      }
   end
   
   MagicComm:RegisterListener(self, "MD")
   self:RegisterEvent("RAID_ROSTER_UPDATE", "StartOrEndRaid")
   self:RegisterEvent("ZONE_CHANGED_NEW_AREA")
   self:ZONE_CHANGED_NEW_AREA()
end


function MagicDKP:TakeStandbyAttendance(event)
   if self.trace then self:trace("Taking standby attendance.") end
   local n = GetNumGuildMembers()
   if not n or n < 1 then return end

   if self.trace then self:trace("Taking standby attendance 3.") end
   self:UnregisterEvent("GUILD_ROSTER_UPDATE") 

   local offline = GetGuildRosterShowOffline()
   local selection = GetGuildRosterSelection()

   SetGuildRosterShowOffline(false)
   SetGuildRosterSelection(0)
   GetGuildRosterInfo(0)

   local numGuildMembers = GetNumGuildMembers()
   for i = 1, numGuildMembers do
      local name, rank, rankIndex = GetGuildRosterInfo(i)
      if not event.attendees[name] then 
	 SetNetworkData("STANDBYCHECK", event.name)
	 self:SendUrgentMessage("WHISPER", name)
	 if self.trace then self:trace("Sending attendance message to %s.", name) end
      end

   end
   SetGuildRosterShowOffline(offline)
   SetGuildRosterSelection(selection)
end


function MagicDKP:OnStandbyResponse(player, eventName)
   local raid = raids[#raids]
   local event = raid.events[eventName]
   if event then
      if not event.attendees[player] then
	 event.standby[player] = true
	 if self.debug then self:debug("Adding %s to event %s.", player, eventName) end
      else
	 if self.trace then self:trace("Not adding %s to standby - already attending.", player) end
      end
   end
end

function MagicDKP:OnDisable()
   self:UnregisterEvent("RAID_ROSTER_UPDATE")
   self:UnregisterEvent("ZONE_CHANGED_NEW_AREA")
   MagicComm:UnregisterListener(self, "MD")
end

local function UnitIsEligable (unit)
   return UnitExists(unit) and UnitCanAttack("player", unit) 
      and UnitClassification(unit) == "worldboss"
      and not UnitPlayerControlled(unit)  and not UnitIsPlayer(unit) -- not needed once classification is used
end


local function GUIDToUID(guid)
   local uid = tonumber(sub(guid, 7, 12), 16)
   if uid == 0 then return end
   return uid
end

local function GetUnitID(unit)
   local guid = UnitGUID(unit)
   return guid, GUIDToUID(guid)
end


function MagicDKP:ScanTarget(unit)
   if not currentZone or not UnitIsEligable(unit) then
      return
   end

   local guid, uid = GetUnitID(unit)

   if uid and not currentZone.mobs[uid] then
      self:AddNewBoss(uid, UnitName(unit), currentZone)
   end
   
   if UnitIsDead(unit) then
      -- we found a dead boss, check to see if this guy is killed
      if not currentEvent and currentRaid then
	 local event = currentZone.mobs[uid].event or currentZone.mobs[uid].name
	 local tmpEvent = currentRaid.events[event] 
	 if tmpEvent then
	    if self.trace then self:trace("Found dead boss mob with finished event - activating.") end
	    currentEvent = tmpEvent
	    self:ScheduleTimer("ClearEvent", 120, currentEvent) -- clear it after 120 seconds
	 elseif not currentZone.mobs[uid].ignore then
	    if self.trace then self:trace("Possibly logging event for [mouseover] %s: %s.", event, tostring(uid)) end
	    self:LogRaidEvent(uid, currentZone.mobs[uid])
	 end
      end
   elseif not loggingDeaths then
      if self.trace then self:trace("Detected boss mob, enabling combat event log filtering.") end 
      self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED") -- eligable mob found, enable death tracking
      loggingDeaths = true
   end
end

function MagicDKP:AddDefaultZones()
   if not MagicDKP.defaultZones then
      return
   end
   local key
   for zone, zonedata in pairs(MagicDKP.defaultZones) do
      key = self:MakeKey(zone)
      if not zones[key] then
	 zones[key] = {
	    name = zone,
	    size = zonedata.size,
	    mobs = { }
	 }
      end
      zone = zones[key].mobs
      for mob, data in pairs(zonedata.mobs) do
	 if not zone[mob] then
	    zone[mob] = {}
	    for key,val in pairs(data) do
	       zone[mob][key] = val
	    end
	 end
      end
   end
end

function MagicDKP:ZONE_CHANGED_NEW_AREA()
   local zone,name = self:GetZoneName()
   currentZone = nil
   if zone == nil or zone == "" then
      self:ScheduleTimer("ZONE_CHANGED_NEW_AREA",5)
   else
      local inInstance, type = IsInInstance()
      self:StartOrEndRaid() -- make sure we have a raid if we should
      if not currentRaid then return end
	 
      if type ~= "raid" then  -- FIXME
	 zone = "WorldBoss"
	 name = "World Boss"
      end
      if not zones[zone] then
	 zones[zone] = {
	    name = name, 
	    mobs = {}
	 }
	 self:BuildConfiguration()
	 self:NotifyChange()
      end

      currentZone = zones[zone]
      if currentRaid and not currentRaid.zone and type == "raid" then
	 currentRaid.zone = name
	 self:NotifyChange()
	 if self.trace then self:trace("Set %s to the zone of the current raid.", name) end
      end
   end
end

local linkpat = '|c........|Hitem:(%%d+):.-|h|r'
local globalpatterns = {
   "LOOT_ITEM",
   "LOOT_ITEM_SELF",
   "LOOT_ITEM_MULTIPLE",
   "LOOT_ITEM_SELF_MULTIPLE",
}

local patterns = {}
for _, global in ipairs(globalpatterns) do
   local pattern = _G[global]
   if sub(pattern, 1,2) == "%s" then
      pattern = "(%a+)"..sub(pattern, 3)
   end
   patterns[global] = gsub(gsub(pattern, "%%d", "(%%d+)"), "%%s", linkpat)
end
local npatterns = #patterns
local GetItemCount, GetItemInfo, match = GetItemCount, GetItemInfo, string.match

function MagicDKP:CHAT_MSG_LOOT(event, msg)
   local looter, itemid, count

   looter, itemid, count = match(msg, patterns["LOOT_ITEM_MULTIPLE"])

   if looter and itemid and count then
      MagicDKP:HandleLoot(looter, tonumber(itemid), tonumber(count))
      return
   end
   
   itemid, count  = match(msg, patterns["LOOT_ITEM_SELF_MULTIPLE"])
   if itemid and count then
      MagicDKP:HandleLoot(playerName, tonumber(itemid), tonumber(count))
      return
   end

   looter, itemid = match(msg, patterns["LOOT_ITEM"])
   if looter and itemid then
      MagicDKP:HandleLoot(looter, tonumber(itemid), 1)
      return
   end

   itemid = match(msg, patterns["LOOT_ITEM_SELF"])
   if itemid then
      MagicDKP:HandleLoot(playerName, tonumber(itemid), 1)
      return
   end
end

function MagicDKP:HandleLoot(looter, itemid, count)

   local name,link,rarity = GetItemInfo(itemid)
   if self.spam then self:spam("handling %s", link) end

   if rarity < 4 and not items[itemid] then
      if self.spam then self:spam("Ignoring %s [low item quality]", link) end
      return
   end

   if not items[itemid] then
      self:UpdateItem(itemid)
   end

   local itemData = items[itemid]

   if itemData.ignore then
      if self.spam then self:spam("Ignoring %s [ignored in database]", link) end
      return
   end
   if self.spam then self:spam("adding %s ", link) end

   if itemData.bank then
      self:AddItem("bank", itemid, 0, count)
   elseif itemData.fixed then
      self:AddItem(looter, itemid, (itemData.value or 0)*count, count)
   else 
      QueueItem(looter, itemid, (itemData.value or 0), link, count)
   end
end
      
function MagicDKP:TakeAttendance()
   local groups = 5 -- FIXME
   if not currentEvent then return end
   currentEvent.eventTime = self:GetGameTimeAsUnix()
   for i = 1, GetNumRaidMembers() do
      local name, _, group = GetRaidRosterInfo(i)
      if group <= groups then
	 currentEvent.attendees[name] = true
	 if self.trace then self:trace("Adding %s to the current event %s.", name, currentEvent.name) end
      end
   end

   self:RegisterEvent("GUILD_ROSTER_UPDATE", "TakeStandbyAttendance", currentEvent)
   GuildRoster()
   
   currentRaid.events[currentEvent.name] = currentEvent
   self:BuildConfiguration()
   self:ScheduleTimer("ClearEvent", 300, currentEvent)
   self:NotifyChange()
end


local raidAttendanceText = L["Record attendance for %s?"]
local eventInProgress

function MagicDKP:LogRaidEvent(uid, unitData)
   if not currentRaid then return end -- should never happen
   local event = unitData.event or unitData.name 
   if not eventInProgress or event ~= eventInProgress.name then
      if currentRaid.events[event] then
	 eventInProgress = currentRaid.events[event]
      else
	 if self.trace then self:trace("Creating a new event...") end
	 eventInProgress = {
	    attendees = {},
	    standby = {},
	    items = {},
	    needed = {},
	    name = event,
	 }
	 for mobuid,data in pairs(currentZone.mobs) do
	    if (data.event or data.name) == event and not data.ignore then
	       eventInProgress.needed[mobuid] = true
	    end
	 end
      end
   end

   eventInProgress.needed[uid] = nil

   if not next(eventInProgress.needed) then 
      eventInProgress.needed = nil -- don't need to save this
      if self.trace then self:trace("Event finished, taking attendance...") end
      if loggingDeaths then
	 loggingDeaths = false
	 -- we're done with this event so no longer need track this
	 self:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED") 
      end
      currentEvent = eventInProgress
      eventInProgress = nil
      currentBoss = nil
      if not currentEvent.eventTime then
	 StaticPopupDialogs["MDKPAttendance"].text = raidAttendanceText:format(event)
	 StaticPopup_Show("MDKPAttendance")
      end
   end
end


function MagicDKP:ClearEvent(event)
   if event == currentEvent then
      currentEvent = nil
   end
end
function MagicDKP:PLAYER_REGEN_DISABLED()
   eventInProgress = nil
   currentBoss = nil
end

function MagicDKP:COMBAT_LOG_EVENT_UNFILTERED(_, _, event, _, _, _, guid)
   local uid
   if not currentZone or not currentRaid then
      self:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
      return
   end
   if not currentBoss then
      uid = GUIDToUID(guid) 
      if currentZone.mobs[uid] and not currentZone.mobs[uid].isfarm  then
	 -- This means we're engaged with a boss that gives wipe dkp
	 if self.trace then self:trace("Engaged boss: %s", currentZone.mobs[uid].name) end
	 currentBoss = uid
      end
   end
   
   if event ~= "UNIT_DIED" and event ~= "PARTY_KILL" then return end

   uid = uid or GUIDToUID(guid)
   
   if uid and currentZone.mobs[uid] and not currentZone.mobs[uid].ignore then
      if self.trace then self:trace("Possibly logging event for %s: %s.", event, tostring(uid)) end
      self:LogRaidEvent(uid, currentZone.mobs[uid])
   end
end

local isInRaid

-- Get the current raid or create a new one if none is active
function MagicDKP:FindOrCreateRaid()
   local raid = raids[#raids]
   local changed = false
   if not raid or raid.endTime then
      raid = {
	 startTime = self:GetGameTimeAsUnix(),
	 events = {
	 },
      }
      raids[#raids+1] = raid
      if self.debug then self:debug("Created new raid.") end
      changed = true
      self:AddRaidConfig(#raids)
   else
      if self.debug then self:debug("Found active raid.") end
   end
   if currentZone and not raid.zone and currentZone.name ~= "World Boss" then
      raid.zone = currentZone.name
      changed = true
   end
   if changed then
      self:NotifyChange()
   end
   currentRaid = raid 
end

function MagicDKP:EndRaid()
   currentRaid = raids[#raids]
   if currentRaid and not currentRaid.endTime  then
      if next(currentRaid.events) then
	 currentRaid.endTime = self:GetGameTimeAsUnix()
	 if self.debug then self:debug("Ending active raid.") end
	 self:NotifyChange()
      else
	 -- No events, don't save
	 self:DeleteRaidConfig(#raids)
	 raids[#raids] = nil
	 if self.debug then self:debug("Deleting active raid (no events).") end
	 self:NotifyChange()
      end
   elseif self.trace then
      self:trace("No active raid found.")
   end
   currentRaid = nil
   currentZone = nil
end

function MagicDKP:StartOrEndRaid()
   local num = GetNumRaidMembers()
   if currentRaid and  currentRaid.endTime then
      currentRaid = nil
   end
   if num >= tonumber(db.raidsize) then
      if not currentRaid then
	 if self.trace then self:trace("No current raid - creating new or finding active. "); end
	 self:FindOrCreateRaid()
      end
      self:RegisterEvent("UPDATE_MOUSEOVER_UNIT", "ScanTarget", "mouseover")   
      self:RegisterEvent("PLAYER_TARGET_CHANGED", "ScanTarget", "target")   
      self:RegisterEvent("CHAT_MSG_LOOT")	    
      self:RegisterEvent("PLAYER_REGEN_DISABLED")
   elseif num == 0 then
      if not currentRaid then
	 currentRaid = raids[#raids]
      end
      if currentRaid and not currentRaid.endTime then
	 if self.trace then self:trace("Current raid should be ended.") end
	 self:EndRaid() -- we're no longer in a raid at all, so end the raid
      end
      self:UnregisterEvent("PLAYER_REGEN_DISABLED")
      self:UnregisterEvent("UPDATE_MOUSEOVER_UNIT")   
      self:UnregisterEvent("PLAYER_TARGET_CHANGED")
      self:UnregisterEvent("CHAT_MSG_LOOT")
   end   
end

function MagicDKP:GetGameTimeAsUnix()
   local h, m = GetGameTime()
   local n = date("*t")
   n.hour = h
   n.min = m
   return time(n)
end

function MagicDKP:FromUnixTime(timestamp, prefix, postfix, short)
   local dt = date("*t", timestamp)
   if short then
      return string.format("%s%02d/%02d %s", prefix or "" , dt.month, dt.day, postfix or "")
   else
      return string.format("%s%02d/%02d/%d %02d:%02d:%02d%s", prefix or "" , dt.month, dt.day, dt.year, dt.hour, dt.min, dt.sec, postfix or "")
   end
end

function MagicDKP:LogCustomEvent(eventid)
   if not currentRaid then
      if self.error then self:error("No active raid - not logging event '%s'", events[eventid]) end
      return
   end
   local oldevent = currentEvent
   currentEvent =  currentRaid.events[events[eventid]]
   if not currentEvent then
      currentEvent = {
	 attendees = {},
	 standby = {},
	 name = events[eventid],
      }
   end

   if not currentEvent.eventTime or not next(currentEvent.attendees) then
      if self.debug then self:debug("Taking Attendance for: %s", events[eventid]) end
      self:TakeAttendance()
   elseif self.trace then
      self:trace("Event %s already logged with attendance.", events[eventid])
   end
   currentEvent = oldevent
end

function MagicDKP:AddWipe()
   if not currentRaid then
      if self.error then self:error("No active raid - not logging wipe.") end
      return
   elseif not currentBoss then
      if self.error then self:error("No boss recently engaged - not logging wipe.") end
      return
   end

   local event = string.format("%s - %s #", L["Wipe"], BL[currentZone.mobs[currentBoss].name])
   local eventnum
   currentBoss = nil
   eventInProgress = nil
   for id = 1, 100 do
      eventnum = event..id
      if not currentRaid.events[eventnum] then
	 currentEvent = {
	    attendees = {},
	    standby = {},
	    name = eventnum,
	 }
	 currentRaid.events[eventnum] = currentEvent
	 if self.debug then self:debug("Logging Wipe: %s", eventnum) end
	 self:TakeAttendance()
	 currentEvent = nil
	 return
      end
   end
end

function MagicDKP:OnProfileChanged(event, newdb)
   if event ~= "OnProfileDeleted" then
      db = self.db.profile
      self:SetLogLevel(db.logLevel)
   end
      
   self:NotifyChange()

end


local exportDialog

function MagicDKP:ExportCTRT(info)
   local itempat = '|c(........)|Hitem:(.-)|h%['
   local raidid = tonumber(type(info) == "table" and info[#info-1] or info)
   if raids[raidid] then
      local raid = raids[raidid]
      local loot = ""
      local lootcount, count = 1, 1
      local zone = raid.zone or "Unknown"
      local export = "<RaidInfo>"
	 .. self:FromUnixTime(raid.startTime, "<key>", "</key>")
	 .. self:FromUnixTime(raid.startTime, "<start>", "</start>")
	 .. self:FromUnixTime(raid.endTime or raid.startTime , "<end>", "</end>")
	 .. "<zone>"..zone.."</zone>"

      local bosskills = ""

      local jointxt, leavetxt = "", ""
      local joincount = 1
      local sortevents = {}
      for id in pairs(raid.events) do
	 sortevents[#sortevents+1] = id
      end
      table.sort(sortevents,
		 function(a, b)
		    return (raid.events[a].eventTime or 0) < (raid.events[b].eventTime or 0)
		 end)
      for _,id in pairs(sortevents) do
	 local data = raid.events[id]
	 local key = "key"..count
	 local attendees = ""
	 local acount = 1
	 local eventTime = data.eventTime or raid.startTime
	 local eventTimeStr = self:FromUnixTime(eventTime)
	 count = count + 1
	 
	 for name in pairs(data.attendees) do
	    local akey = "key"..acount
	    acount = acount + 1
	    attendees = string.format("%s<%s><name>%s</name></%s>", attendees, akey, name, akey)
	    jointxt = string.format("%s<key%d><name>%s</name><time>%s</time></key%s>", jointxt, joincount, name, self:FromUnixTime(eventTime-100), joincount)
	    leavetxt = string.format("%s<key%d><name>%s</name><time>%s</time></key%s>", leavetxt, joincount, name, self:FromUnixTime(eventTime+100), joincount)
	    joincount = joincount + 1
	 end
	 bosskills = string.format("%s<%s><name>%s</name><time>%s</time><attendees>%s</attendees></%s>",
				bosskills, key, id, eventTimeStr, attendees, key)
	 if data.items then
	    for _,item in pairs(data.items) do
	       local itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType,
	       itemSubType, itemStackCount, itemEquipLoc, itemTexture = unpack(items[item.itemid].itemInfo) 
	       --|cffffffff|Hitem:23984:0:0:0:0:0:0:0|h[Irradiated Crystal Shard]|h|
	       if self.spam then self:spam("Exporting item %d : %s", item.itemid, tostring(itemName)) end

	       local color, link = match(itemLink, itempat)
	       local lootkey = "key"..lootcount
	       lootcount = lootcount + 1
	       loot = string.format("%s<%s><ItemName>%s</ItemName><ItemID>%s</ItemID><Icon>%s</Icon><Class>%s</Class><SubClass>%s</SubClass><Color>%s</Color><Count>%d</Count><Player>%s</Player><Costs>%d</Costs><Time>%s</Time><Zone>%s</Zone><Boss>%s</Boss><Note><![CDATA[ - Zone: %s - Boss: %s - %d DKP]]></Note></%s>",
				    loot, lootkey, itemName, link, itemTexture, itemType, itemSubType,color,
				    item.count, item.looter, item.value, eventTimeStr, item.zone or zone, id, item.zone or zone, id, item.value, lootkey)
	       
	    end
	 end
      end
      export = string.format("%s<BossKills>%s</BossKills><Join>%s</Join><Leave>%s</Leave><Loot>%s</Loot></RaidInfo>", export, bosskills, jointxt, leavetxt, loot)
      if not exportDialog then
	 exportDialog = GUI:Create("MDKP_Dialog")
	 exportDialog:SetTitle(L["CTRT Export"])
	 exportDialog:SetLayout("Fill")
	 exportDialog.editbox:SetWidth(350)
	 exportDialog.editbox:SetNumeric(false)
	 exportDialog:SetText(L["Copy the data below to export in CTRT format."])

	 exportDialog.close = GUI:Create("Button")
	       
	 exportDialog:AddChild(exportDialog.close)
	 exportDialog.close:SetText(L["Close"])
	 
	 exportDialog.close:SetCallback("OnClick",
					function()
					   exportDialog:Hide()
					   exportDialog.editbox:SetText("")
					end
				     )
	 
      end
      exportDialog.editbox:SetText(export)
      exportDialog:Show()
   end
end

function MagicDKP:SendUrgentMessage(channel, recipient)
   MagicComm:SendUrgentMessage(networkData, "MD", channel, recipient)
end

function MagicDKP:SendBulkMessage()
   if MagicMarker:IsValidMarker() then
      MagicComm:SendBulkMessage(networkData, "MD")
   end
end

do
   local verRespMsg = "%s: %s revision %s"
   function MagicDKP:OnVersionResponse(ver, major, minor, sender)
      self:Print(verRespMsg:format(sender, major or "Unknown", minor or "Unknown"))
   end
end

function MagicDKP:QueryAddonVersions()
   SetNetworkData("VCHECK")
   self:SendUrgentMessage()
end
